# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel.ui'
#
# Created: Fri Sep 26 10:24:16 2014
#      by: PyQt5 UI code generator 5.3.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(820, 546)
        self.gridLayout_2 = QtWidgets.QGridLayout(Form)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(Form)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.opt_language = QtWidgets.QComboBox(self.tab)
        self.opt_language.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToMinimumContentsLengthWithIcon)
        self.opt_language.setMinimumContentsLength(20)
        self.opt_language.setObjectName("opt_language")
        self.gridLayout_9.addWidget(self.opt_language, 2, 1, 1, 1)
        self.opt_systray_icon = QtWidgets.QCheckBox(self.tab)
        self.opt_systray_icon.setObjectName("opt_systray_icon")
        self.gridLayout_9.addWidget(self.opt_systray_icon, 3, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_9.addItem(spacerItem, 9, 0, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.tab)
        self.label_17.setObjectName("label_17")
        self.gridLayout_9.addWidget(self.label_17, 1, 0, 1, 1)
        self.opt_gui_layout = QtWidgets.QComboBox(self.tab)
        self.opt_gui_layout.setMaximumSize(QtCore.QSize(250, 16777215))
        self.opt_gui_layout.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToMinimumContentsLengthWithIcon)
        self.opt_gui_layout.setMinimumContentsLength(20)
        self.opt_gui_layout.setObjectName("opt_gui_layout")
        self.gridLayout_9.addWidget(self.opt_gui_layout, 1, 1, 1, 1)
        self.opt_disable_animations = QtWidgets.QCheckBox(self.tab)
        self.opt_disable_animations.setObjectName("opt_disable_animations")
        self.gridLayout_9.addWidget(self.opt_disable_animations, 3, 1, 1, 1)
        self.opt_disable_tray_notification = QtWidgets.QCheckBox(self.tab)
        self.opt_disable_tray_notification.setObjectName("opt_disable_tray_notification")
        self.gridLayout_9.addWidget(self.opt_disable_tray_notification, 4, 0, 1, 1)
        self.opt_show_splash_screen = QtWidgets.QCheckBox(self.tab)
        self.opt_show_splash_screen.setObjectName("opt_show_splash_screen")
        self.gridLayout_9.addWidget(self.opt_show_splash_screen, 4, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(self.tab)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.font_display = QtWidgets.QLineEdit(self.tab)
        self.font_display.setReadOnly(True)
        self.font_display.setObjectName("font_display")
        self.horizontalLayout.addWidget(self.font_display)
        self.gridLayout_9.addLayout(self.horizontalLayout, 6, 0, 1, 1)
        self.change_font_button = QtWidgets.QPushButton(self.tab)
        self.change_font_button.setObjectName("change_font_button")
        self.gridLayout_9.addWidget(self.change_font_button, 6, 1, 1, 1)
        self.label_widget_style = QtWidgets.QLabel(self.tab)
        self.label_widget_style.setObjectName("label_widget_style")
        self.gridLayout_9.addWidget(self.label_widget_style, 0, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.opt_toolbar_icon_size = QtWidgets.QComboBox(self.groupBox_2)
        self.opt_toolbar_icon_size.setObjectName("opt_toolbar_icon_size")
        self.gridLayout_8.addWidget(self.opt_toolbar_icon_size, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBox_2)
        self.label_5.setObjectName("label_5")
        self.gridLayout_8.addWidget(self.label_5, 0, 0, 1, 1)
        self.opt_toolbar_text = QtWidgets.QComboBox(self.groupBox_2)
        self.opt_toolbar_text.setObjectName("opt_toolbar_text")
        self.gridLayout_8.addWidget(self.opt_toolbar_text, 1, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.groupBox_2)
        self.label_8.setObjectName("label_8")
        self.gridLayout_8.addWidget(self.label_8, 1, 0, 1, 1)
        self.gridLayout_9.addWidget(self.groupBox_2, 7, 0, 1, 2)
        self.label_7 = QtWidgets.QLabel(self.tab)
        self.label_7.setObjectName("label_7")
        self.gridLayout_9.addWidget(self.label_7, 2, 0, 1, 1)
        self.opt_ui_style = QtWidgets.QComboBox(self.tab)
        self.opt_ui_style.setObjectName("opt_ui_style")
        self.gridLayout_9.addWidget(self.opt_ui_style, 0, 1, 1, 1)
        self.opt_book_list_tooltips = QtWidgets.QCheckBox(self.tab)
        self.opt_book_list_tooltips.setObjectName("opt_book_list_tooltips")
        self.gridLayout_9.addWidget(self.opt_book_list_tooltips, 5, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.tab)
        self.label_4.setObjectName("label_4")
        self.gridLayout_9.addWidget(self.label_4, 8, 0, 1, 1)
        self.opt_book_list_extra_row_spacing = QtWidgets.QSpinBox(self.tab)
        self.opt_book_list_extra_row_spacing.setMinimum(-15)
        self.opt_book_list_extra_row_spacing.setObjectName("opt_book_list_extra_row_spacing")
        self.gridLayout_9.addWidget(self.opt_book_list_extra_row_spacing, 8, 1, 1, 1)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(I("lt.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tab, icon, "")
        self.cover_grid_tab = QtWidgets.QWidget()
        self.cover_grid_tab.setObjectName("cover_grid_tab")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.cover_grid_tab)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget_2 = QtWidgets.QTabWidget(self.cover_grid_tab)
        self.tabWidget_2.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabWidget_2.setObjectName("tabWidget_2")
        self.tab_5 = QtWidgets.QWidget()
        self.tab_5.setObjectName("tab_5")
        self.formLayout_2 = QtWidgets.QFormLayout(self.tab_5)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_19 = QtWidgets.QLabel(self.tab_5)
        self.label_19.setWordWrap(True)
        self.label_19.setObjectName("label_19")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.SpanningRole, self.label_19)
        spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.formLayout_2.setItem(3, QtWidgets.QFormLayout.FieldRole, spacerItem1)
        self.label_14 = QtWidgets.QLabel(self.tab_5)
        self.label_14.setObjectName("label_14")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_14)
        self.opt_field_under_covers_in_grid = QtWidgets.QComboBox(self.tab_5)
        self.opt_field_under_covers_in_grid.setObjectName("opt_field_under_covers_in_grid")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.opt_field_under_covers_in_grid)
        self.label_16 = QtWidgets.QLabel(self.tab_5)
        self.label_16.setObjectName("label_16")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_16)
        self.opt_cover_grid_spacing = QtWidgets.QDoubleSpinBox(self.tab_5)
        self.opt_cover_grid_spacing.setDecimals(2)
        self.opt_cover_grid_spacing.setSingleStep(0.1)
        self.opt_cover_grid_spacing.setObjectName("opt_cover_grid_spacing")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.opt_cover_grid_spacing)
        self.cg_background_box = QtWidgets.QGroupBox(self.tab_5)
        self.cg_background_box.setObjectName("cg_background_box")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.cg_background_box)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.SpanningRole, self.cg_background_box)
        self.groupBox_4 = QtWidgets.QGroupBox(self.tab_5)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_20 = QtWidgets.QLabel(self.groupBox_4)
        self.label_20.setWordWrap(True)
        self.label_20.setObjectName("label_20")
        self.gridLayout_4.addWidget(self.label_20, 0, 0, 1, 5)
        self.cover_grid_aspect_ratio = QtWidgets.QLabel(self.groupBox_4)
        self.cover_grid_aspect_ratio.setText("")
        self.cover_grid_aspect_ratio.setObjectName("cover_grid_aspect_ratio")
        self.gridLayout_4.addWidget(self.cover_grid_aspect_ratio, 3, 0, 1, 5)
        self.label_11 = QtWidgets.QLabel(self.groupBox_4)
        self.label_11.setObjectName("label_11")
        self.gridLayout_4.addWidget(self.label_11, 1, 0, 1, 1)
        self.opt_cover_grid_width = QtWidgets.QDoubleSpinBox(self.groupBox_4)
        self.opt_cover_grid_width.setDecimals(1)
        self.opt_cover_grid_width.setObjectName("opt_cover_grid_width")
        self.gridLayout_4.addWidget(self.opt_cover_grid_width, 1, 1, 1, 1)
        self.cover_grid_larger_cover = QtWidgets.QPushButton(self.groupBox_4)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(I("plus.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cover_grid_larger_cover.setIcon(icon1)
        self.cover_grid_larger_cover.setObjectName("cover_grid_larger_cover")
        self.gridLayout_4.addWidget(self.cover_grid_larger_cover, 1, 2, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.groupBox_4)
        self.label_12.setObjectName("label_12")
        self.gridLayout_4.addWidget(self.label_12, 2, 0, 1, 1)
        self.opt_cover_grid_height = QtWidgets.QDoubleSpinBox(self.groupBox_4)
        self.opt_cover_grid_height.setDecimals(1)
        self.opt_cover_grid_height.setObjectName("opt_cover_grid_height")
        self.gridLayout_4.addWidget(self.opt_cover_grid_height, 2, 1, 1, 1)
        self.cover_grid_smaller_cover = QtWidgets.QPushButton(self.groupBox_4)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(I("minus.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cover_grid_smaller_cover.setIcon(icon2)
        self.cover_grid_smaller_cover.setObjectName("cover_grid_smaller_cover")
        self.gridLayout_4.addWidget(self.cover_grid_smaller_cover, 2, 2, 1, 1)
        self.cover_grid_reset_size = QtWidgets.QPushButton(self.groupBox_4)
        self.cover_grid_reset_size.setObjectName("cover_grid_reset_size")
        self.gridLayout_4.addWidget(self.cover_grid_reset_size, 1, 3, 2, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem2, 1, 4, 1, 1)
        self.formLayout_2.setWidget(10, QtWidgets.QFormLayout.SpanningRole, self.groupBox_4)
        self.opt_emblem_size = QtWidgets.QSpinBox(self.tab_5)
        self.opt_emblem_size.setMinimum(16)
        self.opt_emblem_size.setMaximum(1000)
        self.opt_emblem_size.setObjectName("opt_emblem_size")
        self.formLayout_2.setWidget(11, QtWidgets.QFormLayout.FieldRole, self.opt_emblem_size)
        self.label_21 = QtWidgets.QLabel(self.tab_5)
        self.label_21.setObjectName("label_21")
        self.formLayout_2.setWidget(11, QtWidgets.QFormLayout.LabelRole, self.label_21)
        self.opt_cover_grid_show_title = QtWidgets.QCheckBox(self.tab_5)
        self.opt_cover_grid_show_title.setObjectName("opt_cover_grid_show_title")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.SpanningRole, self.opt_cover_grid_show_title)
        self.label_22 = QtWidgets.QLabel(self.tab_5)
        self.label_22.setObjectName("label_22")
        self.formLayout_2.setWidget(12, QtWidgets.QFormLayout.LabelRole, self.label_22)
        self.opt_emblem_position = QtWidgets.QComboBox(self.tab_5)
        self.opt_emblem_position.setObjectName("opt_emblem_position")
        self.formLayout_2.setWidget(12, QtWidgets.QFormLayout.FieldRole, self.opt_emblem_position)
        self.tabWidget_2.addTab(self.tab_5, "")
        self.emblems_tab = QtWidgets.QWidget()
        self.emblems_tab.setObjectName("emblems_tab")
        self.tabWidget_2.addTab(self.emblems_tab, "")
        self.tab_6 = QtWidgets.QWidget()
        self.tab_6.setObjectName("tab_6")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab_6)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab_6)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout.setObjectName("gridLayout")
        self.opt_cover_grid_disk_cache_size = QtWidgets.QSpinBox(self.groupBox_3)
        self.opt_cover_grid_disk_cache_size.setMinimum(100)
        self.opt_cover_grid_disk_cache_size.setSingleStep(100)
        self.opt_cover_grid_disk_cache_size.setObjectName("opt_cover_grid_disk_cache_size")
        self.gridLayout.addWidget(self.opt_cover_grid_disk_cache_size, 2, 2, 1, 1)
        self.cover_grid_open_cache = QtWidgets.QPushButton(self.groupBox_3)
        self.cover_grid_open_cache.setObjectName("cover_grid_open_cache")
        self.gridLayout.addWidget(self.cover_grid_open_cache, 3, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 1, 3, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.groupBox_3)
        self.label_13.setWordWrap(True)
        self.label_13.setObjectName("label_13")
        self.gridLayout.addWidget(self.label_13, 0, 0, 1, 5)
        self.cover_grid_current_disk_cache = QtWidgets.QLabel(self.groupBox_3)
        self.cover_grid_current_disk_cache.setText("")
        self.cover_grid_current_disk_cache.setObjectName("cover_grid_current_disk_cache")
        self.gridLayout.addWidget(self.cover_grid_current_disk_cache, 2, 3, 1, 2)
        spacerItem4 = QtWidgets.QSpacerItem(310, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem4, 3, 2, 1, 3)
        self.label_15 = QtWidgets.QLabel(self.groupBox_3)
        self.label_15.setObjectName("label_15")
        self.gridLayout.addWidget(self.label_15, 1, 0, 1, 2)
        self.label_18 = QtWidgets.QLabel(self.groupBox_3)
        self.label_18.setObjectName("label_18")
        self.gridLayout.addWidget(self.label_18, 2, 0, 1, 2)
        self.cover_grid_empty_cache = QtWidgets.QPushButton(self.groupBox_3)
        self.cover_grid_empty_cache.setObjectName("cover_grid_empty_cache")
        self.gridLayout.addWidget(self.cover_grid_empty_cache, 3, 0, 1, 1)
        self.opt_cover_grid_cache_size_multiple = QtWidgets.QSpinBox(self.groupBox_3)
        self.opt_cover_grid_cache_size_multiple.setMinimum(2)
        self.opt_cover_grid_cache_size_multiple.setMaximum(100)
        self.opt_cover_grid_cache_size_multiple.setObjectName("opt_cover_grid_cache_size_multiple")
        self.gridLayout.addWidget(self.opt_cover_grid_cache_size_multiple, 1, 2, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem5, 4, 1, 1, 1)
        self.verticalLayout_2.addWidget(self.groupBox_3)
        self.tabWidget_2.addTab(self.tab_6, "")
        self.verticalLayout.addWidget(self.tabWidget_2)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(I("grid.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.cover_grid_tab, icon3, "")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.tab_4)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.label_3 = QtWidgets.QLabel(self.tab_4)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.gridLayout_12.addWidget(self.label_3, 2, 1, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.tab_4)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.df_up_button = QtWidgets.QToolButton(self.groupBox)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(I("arrow-up.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.df_up_button.setIcon(icon4)
        self.df_up_button.setObjectName("df_up_button")
        self.gridLayout_3.addWidget(self.df_up_button, 0, 1, 1, 1)
        self.df_down_button = QtWidgets.QToolButton(self.groupBox)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(I("arrow-down.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.df_down_button.setIcon(icon5)
        self.df_down_button.setObjectName("df_down_button")
        self.gridLayout_3.addWidget(self.df_down_button, 2, 1, 1, 1)
        self.field_display_order = QtWidgets.QListView(self.groupBox)
        self.field_display_order.setAlternatingRowColors(True)
        self.field_display_order.setObjectName("field_display_order")
        self.gridLayout_3.addWidget(self.field_display_order, 0, 0, 3, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem6, 1, 1, 1, 1)
        self.gridLayout_12.addWidget(self.groupBox, 2, 0, 2, 1)
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")
        self.opt_bd_show_cover = QtWidgets.QCheckBox(self.tab_4)
        self.opt_bd_show_cover.setObjectName("opt_bd_show_cover")
        self.hboxlayout.addWidget(self.opt_bd_show_cover)
        self.opt_bd_overlay_cover_size = QtWidgets.QCheckBox(self.tab_4)
        self.opt_bd_overlay_cover_size.setObjectName("opt_bd_overlay_cover_size")
        self.hboxlayout.addWidget(self.opt_bd_overlay_cover_size)
        self.opt_use_roman_numerals_for_series_number = QtWidgets.QCheckBox(self.tab_4)
        self.opt_use_roman_numerals_for_series_number.setChecked(True)
        self.opt_use_roman_numerals_for_series_number.setObjectName("opt_use_roman_numerals_for_series_number")
        self.hboxlayout.addWidget(self.opt_use_roman_numerals_for_series_number)
        spacerItem7 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.hboxlayout.addItem(spacerItem7)
        self.gridLayout_12.addLayout(self.hboxlayout, 1, 0, 1, 2)
        self.hboxlayout1 = QtWidgets.QHBoxLayout()
        self.hboxlayout1.setObjectName("hboxlayout1")
        self.label = QtWidgets.QLabel(self.tab_4)
        self.label.setObjectName("label")
        self.hboxlayout1.addWidget(self.label)
        self.opt_default_author_link = QtWidgets.QLineEdit(self.tab_4)
        self.opt_default_author_link.setObjectName("opt_default_author_link")
        self.hboxlayout1.addWidget(self.opt_default_author_link)
        self.search_library_for_author_button = QtWidgets.QPushButton(self.tab_4)
        self.search_library_for_author_button.setObjectName("search_library_for_author_button")
        self.hboxlayout1.addWidget(self.search_library_for_author_button)
        self.gridLayout_12.addLayout(self.hboxlayout1, 0, 0, 1, 2)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(I("book.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tab_4, icon6, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.formLayout = QtWidgets.QFormLayout(self.tab_2)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label_10 = QtWidgets.QLabel(self.tab_2)
        self.label_10.setObjectName("label_10")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_10)
        self.opt_tags_browser_collapse_at = QtWidgets.QSpinBox(self.tab_2)
        self.opt_tags_browser_collapse_at.setMaximum(10000)
        self.opt_tags_browser_collapse_at.setObjectName("opt_tags_browser_collapse_at")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.opt_tags_browser_collapse_at)
        self.label_8111 = QtWidgets.QLabel(self.tab_2)
        self.label_8111.setObjectName("label_8111")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_8111)
        self.opt_tag_browser_dont_collapse = EditWithComplete(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.opt_tag_browser_dont_collapse.sizePolicy().hasHeightForWidth())
        self.opt_tag_browser_dont_collapse.setSizePolicy(sizePolicy)
        self.opt_tag_browser_dont_collapse.setObjectName("opt_tag_browser_dont_collapse")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.opt_tag_browser_dont_collapse)
        self.label_81 = QtWidgets.QLabel(self.tab_2)
        self.label_81.setObjectName("label_81")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_81)
        self.opt_categories_using_hierarchy = EditWithComplete(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.opt_categories_using_hierarchy.sizePolicy().hasHeightForWidth())
        self.opt_categories_using_hierarchy.setSizePolicy(sizePolicy)
        self.opt_categories_using_hierarchy.setObjectName("opt_categories_using_hierarchy")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.opt_categories_using_hierarchy)
        self.opt_tag_browser_old_look = QtWidgets.QCheckBox(self.tab_2)
        self.opt_tag_browser_old_look.setObjectName("opt_tag_browser_old_look")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.SpanningRole, self.opt_tag_browser_old_look)
        self.opt_tags_browser_partition_method = QtWidgets.QComboBox(self.tab_2)
        self.opt_tags_browser_partition_method.setObjectName("opt_tags_browser_partition_method")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.opt_tags_browser_partition_method)
        self.label_9 = QtWidgets.QLabel(self.tab_2)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_9)
        self.opt_show_avg_rating = QtWidgets.QCheckBox(self.tab_2)
        self.opt_show_avg_rating.setChecked(True)
        self.opt_show_avg_rating.setObjectName("opt_show_avg_rating")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.opt_show_avg_rating)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(I("tags.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tab_2, icon7, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.opt_separate_cover_flow = QtWidgets.QCheckBox(self.tab_3)
        self.opt_separate_cover_flow.setObjectName("opt_separate_cover_flow")
        self.gridLayout_11.addWidget(self.opt_separate_cover_flow, 0, 0, 1, 2)
        self.label_6 = QtWidgets.QLabel(self.tab_3)
        self.label_6.setObjectName("label_6")
        self.gridLayout_11.addWidget(self.label_6, 1, 0, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(690, 283, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_11.addItem(spacerItem8, 7, 0, 1, 2)
        self.opt_cover_flow_queue_length = QtWidgets.QSpinBox(self.tab_3)
        self.opt_cover_flow_queue_length.setObjectName("opt_cover_flow_queue_length")
        self.gridLayout_11.addWidget(self.opt_cover_flow_queue_length, 1, 1, 1, 1)
        self.opt_cb_fullscreen = QtWidgets.QCheckBox(self.tab_3)
        self.opt_cb_fullscreen.setObjectName("opt_cb_fullscreen")
        self.gridLayout_11.addWidget(self.opt_cb_fullscreen, 5, 0, 1, 2)
        self.fs_help_msg = QtWidgets.QLabel(self.tab_3)
        self.fs_help_msg.setStyleSheet("margin-left: 1.5em")
        self.fs_help_msg.setWordWrap(True)
        self.fs_help_msg.setObjectName("fs_help_msg")
        self.gridLayout_11.addWidget(self.fs_help_msg, 6, 0, 1, 2)
        self.opt_cover_browser_reflections = QtWidgets.QCheckBox(self.tab_3)
        self.opt_cover_browser_reflections.setObjectName("opt_cover_browser_reflections")
        self.gridLayout_11.addWidget(self.opt_cover_browser_reflections, 2, 0, 1, 1)
        self.opt_cb_preserve_aspect_ratio = QtWidgets.QCheckBox(self.tab_3)
        self.opt_cb_preserve_aspect_ratio.setObjectName("opt_cb_preserve_aspect_ratio")
        self.gridLayout_11.addWidget(self.opt_cb_preserve_aspect_ratio, 3, 0, 1, 2)
        self.opt_show_rating_in_cover_browser = QtWidgets.QCheckBox(self.tab_3)
        self.opt_show_rating_in_cover_browser.setObjectName("opt_show_rating_in_cover_browser")
        self.gridLayout_11.addWidget(self.opt_show_rating_in_cover_browser, 4, 0, 1, 1)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(I("cover_flow.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tab_3, icon8, "")
        self.gridLayout_2.addWidget(self.tabWidget, 0, 0, 1, 1)
        self.label_17.setBuddy(self.opt_gui_layout)
        self.label_2.setBuddy(self.font_display)
        self.label_widget_style.setBuddy(self.opt_ui_style)
        self.label_5.setBuddy(self.opt_toolbar_icon_size)
        self.label_8.setBuddy(self.opt_toolbar_text)
        self.label_7.setBuddy(self.opt_language)
        self.label_4.setBuddy(self.opt_book_list_extra_row_spacing)
        self.label_14.setBuddy(self.opt_field_under_covers_in_grid)
        self.label_16.setBuddy(self.opt_cover_grid_spacing)
        self.label_11.setBuddy(self.opt_cover_grid_width)
        self.label_12.setBuddy(self.opt_cover_grid_height)
        self.label_21.setBuddy(self.opt_emblem_size)
        self.label_22.setBuddy(self.opt_emblem_position)
        self.label_15.setBuddy(self.opt_cover_grid_cache_size_multiple)
        self.label.setBuddy(self.opt_default_author_link)
        self.label_10.setBuddy(self.opt_tags_browser_collapse_at)
        self.label_8111.setBuddy(self.opt_tag_browser_dont_collapse)
        self.label_81.setBuddy(self.opt_categories_using_hierarchy)
        self.label_9.setBuddy(self.opt_tags_browser_partition_method)
        self.label_6.setBuddy(self.opt_cover_flow_queue_length)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(0)
        self.tabWidget_2.setCurrentIndex(0)
        self.opt_cover_grid_show_title.toggled['bool'].connect(self.opt_field_under_covers_in_grid.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.opt_systray_icon.setText(_("Enable system &tray icon (needs restart)"))
        self.label_17.setText(_("User Interface &layout (needs restart):"))
        self.opt_disable_animations.setToolTip(_("Disable all animations. Useful if you have a slow/old computer."))
        self.opt_disable_animations.setText(_("Disable &animations"))
        self.opt_disable_tray_notification.setText(_("Disable &notifications in system tray"))
        self.opt_show_splash_screen.setText(_("Show &splash screen at startup"))
        self.label_2.setText(_("Interface font:"))
        self.change_font_button.setText(_("Change &font (needs restart)"))
        self.label_widget_style.setText(_("User interface &style (needs restart):"))
        self.groupBox_2.setTitle(_("&Toolbar"))
        self.label_5.setText(_("&Icon size:"))
        self.label_8.setText(_("Show &text under icons:"))
        self.label_7.setText(_("Choose &language (requires restart):"))
        self.opt_book_list_tooltips.setText(_("Show &tooltips in the book list"))
        self.label_4.setText(_("Extra &spacing to add between rows in the book list (can be negative):"))
        self.opt_book_list_extra_row_spacing.setSuffix(_(" px"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _("Main Interface"))
        self.label_19.setText(_("Control the cover grid view. You can enable this view by clicking the grid button in the bottom right corner of the main calibre window."))
        self.label_14.setText(_("&Field to show under the covers:"))
        self.label_16.setText(_("&Spacing between covers:"))
        self.opt_cover_grid_spacing.setToolTip(_("The spacing between covers. A value of zero means calculate automatically based on cover size."))
        self.opt_cover_grid_spacing.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_spacing.setSuffix(_(" cm"))
        self.cg_background_box.setTitle(_("Background for the cover grid"))
        self.groupBox_4.setTitle(_("Cover size"))
        self.label_20.setText(_("By default, calibre chooses a cover size based on your computer\'s screen size. You can change the cover size here:"))
        self.label_11.setText(_("Cover &Width: "))
        self.opt_cover_grid_width.setToolTip(_("The width of displayed covers.\n"
"A value of zero means calculate automatically."))
        self.opt_cover_grid_width.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_width.setSuffix(_(" cm"))
        self.cover_grid_larger_cover.setToolTip(_("Make the covers larger, maintaining current aspect ratio."))
        self.cover_grid_larger_cover.setText(_("&Larger covers"))
        self.label_12.setText(_("Cover &Height: "))
        self.opt_cover_grid_height.setToolTip(_("The height of displayed covers.\n"
"A value of zero means calculate automatically."))
        self.opt_cover_grid_height.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_height.setSuffix(_(" cm"))
        self.cover_grid_smaller_cover.setToolTip(_("Make the covers smaller, maintaining current aspect ratio."))
        self.cover_grid_smaller_cover.setText(_("&Smaller covers"))
        self.cover_grid_reset_size.setToolTip(_("Reset size to automatic"))
        self.cover_grid_reset_size.setText(_("&Reset size"))
        self.opt_emblem_size.setSuffix(_(" px"))
        self.label_21.setText(_("&Size of the emblems (if any) shown next to the covers: "))
        self.opt_cover_grid_show_title.setText(_("Show a field (such as title) &under the covers"))
        self.label_22.setText(_("&Location of the emblems shown next to the covers:"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_5), _("Layout"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.emblems_tab), _("Emblems"))
        self.groupBox_3.setTitle(_("Caching of covers for improved performance"))
        self.opt_cover_grid_disk_cache_size.setSpecialValueText(_("Disable"))
        self.opt_cover_grid_disk_cache_size.setSuffix(_(" MB"))
        self.cover_grid_open_cache.setText(_("Open cache directory"))
        self.label_13.setText(_("There are two kinds of caches that calibre uses to improve performance when rendering covers in the grid view. A disk cache that is kept on your hard disk and stores the cover thumbnails and an in memory cache used to ensure flicker free rendering of covers. For best results, keep the memory cache small and the disk cache large, unless you have a lot of extra RAM in your computer and dont mind it being used by the memory cache."))
        self.label_15.setText(_("Number of screenfulls of covers to cache in &memory (keep this small):"))
        self.label_18.setText(_("Maximum amount of disk space to use for caching thumbnails: "))
        self.cover_grid_empty_cache.setText(_("Empty disk cache"))
        self.opt_cover_grid_cache_size_multiple.setToolTip(_("The maximum number of screenfulls of thumbnails to keep in memory. Increasing this will make rendering faster, at the cost of more memory usage. Note that regardless of this setting, a minimum of one hundred thumbnails are always kept in memory, to ensure flicker free rendering."))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_6), _("Performance"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cover_grid_tab), _("Cover Grid"))
        self.label_3.setText(_("Note that <b>comments</b> will always be displayed at the end, regardless of the position you assign here."))
        self.groupBox.setTitle(_("Select displayed metadata"))
        self.df_up_button.setToolTip(_("Move up"))
        self.df_down_button.setToolTip(_("Move down"))
        self.opt_bd_show_cover.setText(_("Show &cover in the book details panel"))
        self.opt_bd_overlay_cover_size.setToolTip(_("Show the size of the book\'s cover in pixels"))
        self.opt_bd_overlay_cover_size.setText(_("Show cover &size"))
        self.opt_use_roman_numerals_for_series_number.setText(_("Use &Roman numerals for series"))
        self.label.setText(_("Default author link template:"))
        self.opt_default_author_link.setToolTip(_("<p>Enter a template to be used to create a link for\n"
"an author in the books information dialog. This template will\n"
"be used when no link has been provided for the author using\n"
"Manage Authors. You can use the values {author} and\n"
"{author_sort}, and any template function."))
        self.search_library_for_author_button.setToolTip(_("Search the calibre library for the author, instead of opening a link"))
        self.search_library_for_author_button.setText(_("Search &calibre library for author"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_4), _("Book Details"))
        self.label_10.setText(_("&Collapse when more items than:"))
        self.opt_tags_browser_collapse_at.setToolTip(_("If a Tag Browser category has more than this number of items, it is divided\n"
"up into subcategories. If the partition method is set to disable, this value is ignored."))
        self.label_8111.setText(_("Categories not to partition:"))
        self.opt_tag_browser_dont_collapse.setToolTip(_("A comma-separated list of categories that are not to\n"
"be partitioned even if the number of items is larger than\n"
"the value shown above. This option can be used to\n"
"avoid collapsing hierarchical categories that have only\n"
"a few top-level elements."))
        self.label_81.setText(_("Categories with &hierarchical items:"))
        self.opt_categories_using_hierarchy.setToolTip(_("A comma-separated list of categories in which items containing\n"
"periods are displayed in the tag browser trees. For example, if\n"
"this box contains \'tags\' then tags of the form \'Mystery.English\'\n"
"and \'Mystery.Thriller\' will be displayed with English and Thriller\n"
"both under \'Mystery\'. If \'tags\' is not in this box,\n"
"then the tags will be displayed each on their own line."))
        self.opt_tag_browser_old_look.setText(_("Use &alternating row colors in the Tag Browser"))
        self.opt_tags_browser_partition_method.setToolTip(_("Choose how tag browser subcategories are displayed when\n"
"there are more items than the limit. Select by first\n"
"letter to see an A, B, C list. Choose partitioned to\n"
"have a list of fixed-sized groups. Set to disabled\n"
"if you never want subcategories"))
        self.label_9.setText(_("Tags browser category &partitioning method:"))
        self.opt_show_avg_rating.setText(_("Show &average ratings in the tags browser"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _("Tag Browser"))
        self.opt_separate_cover_flow.setText(_("Show cover &browser in a separate window (needs restart)"))
        self.label_6.setText(_("&Number of covers to show in browse mode (needs restart):"))
        self.opt_cb_fullscreen.setText(_("When showing cover browser in separate window, show it &fullscreen"))
        self.fs_help_msg.setText(_("You can press the %s keys to toggle full screen mode."))
        self.opt_cover_browser_reflections.setText(_("Show &reflections in the cover browser"))
        self.opt_cb_preserve_aspect_ratio.setToolTip(_("Show covers in their original aspect ratio instead of resizing\n"
"them to all have the same width and height"))
        self.opt_cb_preserve_aspect_ratio.setText(_("Preserve &aspect ratio of covers displayed in the cover browser"))
        self.opt_show_rating_in_cover_browser.setText(_("Show book &rating in cover browser"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), _("Cover Browser"))

from calibre.gui2.complete2 import EditWithComplete

