body {
    background-color: white;
}

#banner {
    position: absolute;
    left: 5px; top: 0px;
}

/*
Search bar
*/
#search_box {
    width: 201px;
    height: 31px;
    background: url(bg_search_box.png);
    top: 5px; right: 20px;
    position: absolute;
}
#search_box #s {
    float: left;
    padding: 0;
    margin: 6px 0 0 6px;
    border-width: 0px;
    font-size: 16px;
    width: 159px;
    background: transparent;
}
#search_box #go {
    float: right;
    margin: 3px 4px 0 0;
}

/*
Count bar
*/
#count_bar {
    position: absolute;
    right: 30px;
    top: 80px;
    font-size:smaller;
    padding-bottom: 5px;
}

#count_bar * img {
    cursor: pointer;
}

#count { cursor: default;}

/*
Styles for the book list
*/
#main {
    width:95%;
    overflow: auto;
    border: solid thin black;
    position: absolute;
    top: 115px; left: 10px;
    z-index: 1;
}

table#book_list  thead tr td {
    width: 100%; 
    padding-right: 1em; padding-left: 1em;
    text-align: center;
    font-weight: bold;
    font-size: 130%;
    border-bottom: thick solid black;
    border-top: thick solid black;
    cursor: pointer;
    font-family: serif;
    padding-top: 0.5ex; padding-bottom: 0.5ex;
}

table#book_list tbody tr td {
    padding-right: 1em; padding-left: 1em;
    /*border-bottom: thin solid black;*/
    padding-bottom: 0.7ex; padding-top: 0.7ex;
    margin: 0pt;
    cursor: pointer;
    
}

table#book_list * .sort_indicator {
    visibility:hidden;
    color: #9f9f9f;
}

table#book_list * .rating {
    color: #3fbbe4;
}

table#book_list * span.subtitle {
    font-size: smaller;
}

table#book_list * a.format {
    text-decoration: none;
    color: blue;
    font-family: monospace;
}

table#book_list * a.format:hover {
    color: red;
}

table#book_list * a.format:visited {
    color: blue;
}

table#book_list * .comments {
    font-size: smaller;
    display: none;
}
/*
Loading message
*/
#loading {
    top: 10px; left: 10px;
    position: absolute;
    font-size: 160%; font-family: monospace;
    text-align: center;
    visibility: hidden;
    z-index: 10000;
    background-color: #aaaaaa;
    opacity: 0.8;
    
}

#loading div {
    top: 50%; position: relative;
}

#cover_pane {
    overflow: auto;
    position: absolute;
    visibility: hidden;
    text-align: right;
    z-index: 2;
    margin: 0pt; padding: 0pt; border-width: 0pt;
}
