# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/tweaks.ui'
#
# Created: Sat Aug  9 08:52:10 2014
#      by: PyQt5 UI code generator 5.3.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(756, 531)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_18 = QtWidgets.QLabel(Form)
        self.label_18.setWordWrap(True)
        self.label_18.setObjectName("label_18")
        self.verticalLayout_4.addWidget(self.label_18)
        self.splitter = QtWidgets.QSplitter(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(10)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tweaks_view = QtWidgets.QListView(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tweaks_view.sizePolicy().hasHeightForWidth())
        self.tweaks_view.setSizePolicy(sizePolicy)
        self.tweaks_view.setMinimumSize(QtCore.QSize(300, 0))
        self.tweaks_view.setAlternatingRowColors(True)
        self.tweaks_view.setSpacing(5)
        self.tweaks_view.setUniformItemSizes(True)
        self.tweaks_view.setObjectName("tweaks_view")
        self.verticalLayout_2.addWidget(self.tweaks_view)
        self.plugin_tweaks_button = QtWidgets.QPushButton(self.layoutWidget)
        self.plugin_tweaks_button.setObjectName("plugin_tweaks_button")
        self.verticalLayout_2.addWidget(self.plugin_tweaks_button)
        self.layoutWidget1 = QtWidgets.QWidget(self.splitter)
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.layoutWidget1)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.groupBox = QtWidgets.QGroupBox(self.layoutWidget1)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.help = QtWidgets.QPlainTextEdit(self.groupBox)
        self.help.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.help.setReadOnly(True)
        self.help.setObjectName("help")
        self.verticalLayout.addWidget(self.help)
        self.gridLayout_3.addWidget(self.groupBox, 1, 0, 1, 3)
        self.groupBox_2 = QtWidgets.QGroupBox(self.layoutWidget1)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName("gridLayout")
        self.edit_tweak = QtWidgets.QPlainTextEdit(self.groupBox_2)
        self.edit_tweak.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.edit_tweak.setObjectName("edit_tweak")
        self.gridLayout.addWidget(self.edit_tweak, 0, 0, 1, 2)
        self.restore_default_button = QtWidgets.QPushButton(self.groupBox_2)
        self.restore_default_button.setObjectName("restore_default_button")
        self.gridLayout.addWidget(self.restore_default_button, 1, 0, 1, 1)
        self.apply_button = QtWidgets.QPushButton(self.groupBox_2)
        self.apply_button.setObjectName("apply_button")
        self.gridLayout.addWidget(self.apply_button, 1, 1, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_2, 2, 0, 1, 3)
        self.search = SearchBox2(self.layoutWidget1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.search.sizePolicy().hasHeightForWidth())
        self.search.setSizePolicy(sizePolicy)
        self.search.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToMinimumContentsLength)
        self.search.setMinimumContentsLength(10)
        self.search.setObjectName("search")
        self.gridLayout_3.addWidget(self.search, 0, 0, 1, 1)
        self.next_button = QtWidgets.QPushButton(self.layoutWidget1)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(I("arrow-down.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.next_button.setIcon(icon)
        self.next_button.setObjectName("next_button")
        self.gridLayout_3.addWidget(self.next_button, 0, 1, 1, 1)
        self.previous_button = QtWidgets.QPushButton(self.layoutWidget1)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(I("arrow-up.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.previous_button.setIcon(icon1)
        self.previous_button.setObjectName("previous_button")
        self.gridLayout_3.addWidget(self.previous_button, 0, 2, 1, 1)
        self.verticalLayout_4.addWidget(self.splitter)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.label_18.setText(_("Values for the tweaks are shown below. Edit them to change the behavior of calibre. Your changes will only take effect <b>after a restart</b> of calibre."))
        self.plugin_tweaks_button.setToolTip(_("Edit tweaks for any custom plugins you have installed"))
        self.plugin_tweaks_button.setText(_("&Plugin tweaks"))
        self.groupBox.setTitle(_("Help"))
        self.groupBox_2.setTitle(_("Edit tweak"))
        self.restore_default_button.setToolTip(_("Restore this tweak to its default value"))
        self.restore_default_button.setText(_("&Reset this tweak"))
        self.apply_button.setToolTip(_("Apply any changes you made to this tweak"))
        self.apply_button.setText(_("&Apply changes to this tweak"))
        self.next_button.setText(_("&Next"))
        self.previous_button.setText(_("&Previous"))

from calibre.gui2.search_box import SearchBox2

