# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/create_custom_column.ui'
#
# Created: Fri Aug  8 18:32:27 2014
#      by: PyQt5 UI code generator 5.3.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_QCreateCustomColumn(object):
    def setupUi(self, QCreateCustomColumn):
        QCreateCustomColumn.setObjectName("QCreateCustomColumn")
        QCreateCustomColumn.setWindowModality(QtCore.Qt.ApplicationModal)
        QCreateCustomColumn.resize(831, 344)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(QCreateCustomColumn.sizePolicy().hasHeightForWidth())
        QCreateCustomColumn.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(I("column.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        QCreateCustomColumn.setWindowIcon(icon)
        self.verticalLayout = QtWidgets.QVBoxLayout(QCreateCustomColumn)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout_2.setContentsMargins(5, 5, 5, 5)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(QCreateCustomColumn)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.label = QtWidgets.QLabel(QCreateCustomColumn)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.column_name_box = QtWidgets.QLineEdit(QCreateCustomColumn)
        self.column_name_box.setMinimumSize(QtCore.QSize(20, 0))
        self.column_name_box.setObjectName("column_name_box")
        self.gridLayout.addWidget(self.column_name_box, 0, 2, 1, 1)
        self.column_heading_box = QtWidgets.QLineEdit(QCreateCustomColumn)
        self.column_heading_box.setObjectName("column_heading_box")
        self.gridLayout.addWidget(self.column_heading_box, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(QCreateCustomColumn)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.column_type_box = QtWidgets.QComboBox(QCreateCustomColumn)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.column_type_box.sizePolicy().hasHeightForWidth())
        self.column_type_box.setSizePolicy(sizePolicy)
        self.column_type_box.setMinimumSize(QtCore.QSize(70, 0))
        self.column_type_box.setObjectName("column_type_box")
        self.horizontalLayout_3.addWidget(self.column_type_box)
        self.use_decorations = QtWidgets.QCheckBox(QCreateCustomColumn)
        self.use_decorations.setObjectName("use_decorations")
        self.horizontalLayout_3.addWidget(self.use_decorations)
        self.is_names = QtWidgets.QCheckBox(QCreateCustomColumn)
        self.is_names.setObjectName("is_names")
        self.horizontalLayout_3.addWidget(self.is_names)
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout_3, 2, 2, 1, 1)
        self.horizontalLayout_31 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_31.setObjectName("horizontalLayout_31")
        self.date_format_box = QtWidgets.QLineEdit(QCreateCustomColumn)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.date_format_box.sizePolicy().hasHeightForWidth())
        self.date_format_box.setSizePolicy(sizePolicy)
        self.date_format_box.setObjectName("date_format_box")
        self.horizontalLayout_31.addWidget(self.date_format_box)
        self.number_format_box = QtWidgets.QLineEdit(QCreateCustomColumn)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.number_format_box.sizePolicy().hasHeightForWidth())
        self.number_format_box.setSizePolicy(sizePolicy)
        self.number_format_box.setObjectName("number_format_box")
        self.horizontalLayout_31.addWidget(self.number_format_box)
        self.date_format_default_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.date_format_default_label.setObjectName("date_format_default_label")
        self.horizontalLayout_31.addWidget(self.date_format_default_label)
        self.number_format_default_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.number_format_default_label.setOpenExternalLinks(True)
        self.number_format_default_label.setObjectName("number_format_default_label")
        self.horizontalLayout_31.addWidget(self.number_format_default_label)
        self.gridLayout.addLayout(self.horizontalLayout_31, 4, 2, 1, 1)
        self.date_format_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.date_format_label.setObjectName("date_format_label")
        self.gridLayout.addWidget(self.date_format_label, 4, 0, 1, 1)
        self.number_format_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.number_format_label.setObjectName("number_format_label")
        self.gridLayout.addWidget(self.number_format_label, 4, 0, 1, 1)
        self.composite_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.composite_label.setObjectName("composite_label")
        self.gridLayout.addWidget(self.composite_label, 5, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.composite_box = QtWidgets.QLineEdit(QCreateCustomColumn)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.composite_box.sizePolicy().hasHeightForWidth())
        self.composite_box.setSizePolicy(sizePolicy)
        self.composite_box.setObjectName("composite_box")
        self.horizontalLayout_4.addWidget(self.composite_box)
        self.composite_default_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.composite_default_label.setObjectName("composite_default_label")
        self.horizontalLayout_4.addWidget(self.composite_default_label)
        self.gridLayout.addLayout(self.horizontalLayout_4, 5, 2, 1, 1)
        self.composite_layout = QtWidgets.QHBoxLayout()
        self.composite_layout.setObjectName("composite_layout")
        self.composite_sort_by_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.composite_sort_by_label.setObjectName("composite_sort_by_label")
        self.composite_layout.addWidget(self.composite_sort_by_label)
        self.composite_sort_by = QtWidgets.QComboBox(QCreateCustomColumn)
        self.composite_sort_by.setObjectName("composite_sort_by")
        self.composite_layout.addWidget(self.composite_sort_by)
        self.composite_make_category = QtWidgets.QCheckBox(QCreateCustomColumn)
        self.composite_make_category.setObjectName("composite_make_category")
        self.composite_layout.addWidget(self.composite_make_category)
        self.composite_contains_html = QtWidgets.QCheckBox(QCreateCustomColumn)
        self.composite_contains_html.setObjectName("composite_contains_html")
        self.composite_layout.addWidget(self.composite_contains_html)
        spacerItem1 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.composite_layout.addItem(spacerItem1)
        self.gridLayout.addLayout(self.composite_layout, 6, 2, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem2, 11, 0, 1, 4)
        self.enum_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.enum_label.setObjectName("enum_label")
        self.gridLayout.addWidget(self.enum_label, 6, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QGridLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.enum_box = QtWidgets.QLineEdit(QCreateCustomColumn)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.enum_box.sizePolicy().hasHeightForWidth())
        self.enum_box.setSizePolicy(sizePolicy)
        self.enum_box.setObjectName("enum_box")
        self.horizontalLayout_2.addWidget(self.enum_box, 0, 0, 1, 1)
        self.enum_default_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.enum_default_label.setObjectName("enum_default_label")
        self.horizontalLayout_2.addWidget(self.enum_default_label, 0, 1, 1, 1)
        self.enum_colors = QtWidgets.QLineEdit(QCreateCustomColumn)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.enum_colors.sizePolicy().hasHeightForWidth())
        self.enum_colors.setSizePolicy(sizePolicy)
        self.enum_colors.setObjectName("enum_colors")
        self.horizontalLayout_2.addWidget(self.enum_colors, 1, 0, 1, 1)
        self.enum_colors_label = QtWidgets.QLabel(QCreateCustomColumn)
        self.enum_colors_label.setObjectName("enum_colors_label")
        self.horizontalLayout_2.addWidget(self.enum_colors_label, 1, 1, 1, 1)
        self.gridLayout.addLayout(self.horizontalLayout_2, 6, 2, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 5, 0, 1, 1)
        self.button_box = QtWidgets.QDialogButtonBox(QCreateCustomColumn)
        self.button_box.setOrientation(QtCore.Qt.Horizontal)
        self.button_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.button_box.setCenterButtons(True)
        self.button_box.setObjectName("button_box")
        self.gridLayout_2.addWidget(self.button_box, 14, 0, 1, 1)
        self.heading_label = QtWidgets.QLabel(QCreateCustomColumn)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.heading_label.setFont(font)
        self.heading_label.setText("")
        self.heading_label.setObjectName("heading_label")
        self.gridLayout_2.addWidget(self.heading_label, 1, 0, 1, 1)
        self.shortcuts = QtWidgets.QLabel(QCreateCustomColumn)
        self.shortcuts.setText("")
        self.shortcuts.setWordWrap(True)
        self.shortcuts.setObjectName("shortcuts")
        self.gridLayout_2.addWidget(self.shortcuts, 3, 0, 1, 1)
        self.line = QtWidgets.QFrame(QCreateCustomColumn)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 2, 0, 1, 1)
        self.line_2 = QtWidgets.QFrame(QCreateCustomColumn)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_2.addWidget(self.line_2, 4, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_2)
        self.label_2.setBuddy(self.column_name_box)
        self.label.setBuddy(self.column_heading_box)
        self.label_3.setBuddy(self.column_type_box)
        self.date_format_label.setBuddy(self.date_format_box)
        self.number_format_label.setBuddy(self.number_format_box)
        self.composite_label.setBuddy(self.composite_box)
        self.composite_sort_by_label.setBuddy(self.composite_sort_by)
        self.enum_label.setBuddy(self.enum_box)

        self.retranslateUi(QCreateCustomColumn)
        QtCore.QMetaObject.connectSlotsByName(QCreateCustomColumn)
        QCreateCustomColumn.setTabOrder(self.column_name_box, self.column_heading_box)
        QCreateCustomColumn.setTabOrder(self.column_heading_box, self.column_type_box)
        QCreateCustomColumn.setTabOrder(self.column_type_box, self.date_format_box)
        QCreateCustomColumn.setTabOrder(self.date_format_box, self.composite_box)
        QCreateCustomColumn.setTabOrder(self.composite_box, self.button_box)

    def retranslateUi(self, QCreateCustomColumn):

        self.label_2.setText(_("&Lookup name"))
        self.label.setText(_("Column &heading"))
        self.column_name_box.setToolTip(_("Used for searching the column. Must contain only digits and lower case letters."))
        self.column_heading_box.setToolTip(_("Column heading in the library view and category name in the tag browser"))
        self.label_3.setText(_("&Column type"))
        self.column_type_box.setToolTip(_("What kind of information will be kept in the column."))
        self.use_decorations.setToolTip(_("Show check marks in the GUI. Values of \'yes\', \'checked\', and \'true\'\n"
"will show a green check. Values of \'no\', \'unchecked\', and \'false\' will show a red X.\n"
"Everything else will show nothing."))
        self.use_decorations.setText(_("Show checkmarks"))
        self.is_names.setToolTip(_("Check this box if this column contains names, like the authors column."))
        self.is_names.setText(_("Contains names"))
        self.date_format_box.setToolTip(_("<p>Date format. Use 1-4 \'d\'s for day, 1-4 \'M\'s for month, and 2 or 4 \'y\'s for year.</p>\n"
"<p>For example:\n"
"<ul>\n"
"<li> ddd, d MMM yyyy gives Mon, 5 Jan 2010<li>\n"
"<li>dd MMMM yy gives 05 January 10</li>\n"
"</ul> "))
        self.date_format_default_label.setToolTip(_("Use MMM yyyy for month + year, yyyy for year only"))
        self.date_format_default_label.setText(_("Default: dd MMM yyyy."))
        self.number_format_default_label.setToolTip(_("<p>The format specifier must begin with <code>{0:</code>\n"
"and end with <code>}</code> You can have text before and after the format specifier.\n"
"            "))
        self.number_format_default_label.setText(_("<p>Default: Not formatted. For format language details see <a href=\"http://docs.python.org/library/string.html#format-string-syntax\">the python documentation</a>"))
        self.date_format_label.setText(_("Format for &dates"))
        self.number_format_label.setText(_("Format for &numbers"))
        self.composite_label.setText(_("&Template"))
        self.composite_box.setToolTip(_("Field template. Uses the same syntax as save templates."))
        self.composite_default_label.setToolTip(_("Similar to save templates. For example, {title} {isbn}"))
        self.composite_default_label.setText(_("Default: (nothing)"))
        self.composite_sort_by_label.setText(_("&Sort/search column by"))
        self.composite_sort_by.setToolTip(_("How this column should handled in the GUI when sorting and searching"))
        self.composite_make_category.setToolTip(_("If checked, this column will appear in the tags browser as a category"))
        self.composite_make_category.setText(_("Show in tags browser"))
        self.composite_contains_html.setText(_("Show as HTML in book details"))
        self.enum_label.setText(_("Values"))
        self.enum_box.setToolTip(_("A comma-separated list of permitted values. The empty value is always\n"
"included, and is the default. For example, the list \'one,two,three\' has\n"
"four values, the first of them being the empty value."))
        self.enum_default_label.setToolTip(_("The empty string is always the first value"))
        self.enum_default_label.setText(_("Values"))
        self.enum_colors.setToolTip(_("A list of color names to use when displaying an item. The\n"
"list must be empty or contain a color for each value."))
        self.enum_colors_label.setText(_("Colors"))


