# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/convert/pdf_output.ui'
#
# Created: Fri Aug  8 18:32:27 2014
#      by: PyQt5 UI code generator 5.3.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(638, 498)
        self.formLayout = QtWidgets.QFormLayout(Form)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label_10 = QtWidgets.QLabel(Form)
        self.label_10.setWordWrap(True)
        self.label_10.setObjectName("label_10")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.SpanningRole, self.label_10)
        self.opt_override_profile_size = QtWidgets.QCheckBox(Form)
        self.opt_override_profile_size.setObjectName("opt_override_profile_size")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.SpanningRole, self.opt_override_profile_size)
        self.label = QtWidgets.QLabel(Form)
        self.label.setObjectName("label")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label)
        self.opt_paper_size = QtWidgets.QComboBox(Form)
        self.opt_paper_size.setObjectName("opt_paper_size")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.opt_paper_size)
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.opt_custom_size = QtWidgets.QLineEdit(Form)
        self.opt_custom_size.setObjectName("opt_custom_size")
        self.horizontalLayout.addWidget(self.opt_custom_size)
        self.label_11 = QtWidgets.QLabel(Form)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout.addWidget(self.label_11)
        self.opt_unit = QtWidgets.QComboBox(Form)
        self.opt_unit.setObjectName("opt_unit")
        self.horizontalLayout.addWidget(self.opt_unit)
        self.formLayout.setLayout(3, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout)
        self.opt_preserve_cover_aspect_ratio = QtWidgets.QCheckBox(Form)
        self.opt_preserve_cover_aspect_ratio.setObjectName("opt_preserve_cover_aspect_ratio")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.SpanningRole, self.opt_preserve_cover_aspect_ratio)
        self.opt_pdf_page_numbers = QtWidgets.QCheckBox(Form)
        self.opt_pdf_page_numbers.setObjectName("opt_pdf_page_numbers")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.SpanningRole, self.opt_pdf_page_numbers)
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.opt_pdf_serif_family = QtWidgets.QFontComboBox(Form)
        self.opt_pdf_serif_family.setObjectName("opt_pdf_serif_family")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_serif_family)
        self.label_5 = QtWidgets.QLabel(Form)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.opt_pdf_sans_family = QtWidgets.QFontComboBox(Form)
        self.opt_pdf_sans_family.setObjectName("opt_pdf_sans_family")
        self.formLayout.setWidget(11, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_sans_family)
        self.label_6 = QtWidgets.QLabel(Form)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.opt_pdf_mono_family = QtWidgets.QFontComboBox(Form)
        self.opt_pdf_mono_family.setObjectName("opt_pdf_mono_family")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_mono_family)
        self.label_7 = QtWidgets.QLabel(Form)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.opt_pdf_standard_font = QtWidgets.QComboBox(Form)
        self.opt_pdf_standard_font.setObjectName("opt_pdf_standard_font")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_standard_font)
        self.label_8 = QtWidgets.QLabel(Form)
        self.label_8.setObjectName("label_8")
        self.formLayout.setWidget(14, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.opt_pdf_default_font_size = QtWidgets.QSpinBox(Form)
        self.opt_pdf_default_font_size.setObjectName("opt_pdf_default_font_size")
        self.formLayout.setWidget(14, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_default_font_size)
        self.label_9 = QtWidgets.QLabel(Form)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(15, QtWidgets.QFormLayout.LabelRole, self.label_9)
        self.opt_pdf_mono_font_size = QtWidgets.QSpinBox(Form)
        self.opt_pdf_mono_font_size.setObjectName("opt_pdf_mono_font_size")
        self.formLayout.setWidget(15, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_mono_font_size)
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setObjectName("groupBox")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBox)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setWordWrap(True)
        self.label_2.setOpenExternalLinks(True)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.SpanningRole, self.label_2)
        self.label_12 = QtWidgets.QLabel(self.groupBox)
        self.label_12.setObjectName("label_12")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_12)
        self.opt_pdf_header_template = QtWidgets.QLineEdit(self.groupBox)
        self.opt_pdf_header_template.setObjectName("opt_pdf_header_template")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_header_template)
        self.label_13 = QtWidgets.QLabel(self.groupBox)
        self.label_13.setObjectName("label_13")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_13)
        self.opt_pdf_footer_template = QtWidgets.QLineEdit(self.groupBox)
        self.opt_pdf_footer_template.setObjectName("opt_pdf_footer_template")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.opt_pdf_footer_template)
        self.formLayout.setWidget(16, QtWidgets.QFormLayout.SpanningRole, self.groupBox)
        self.opt_pdf_add_toc = QtWidgets.QCheckBox(Form)
        self.opt_pdf_add_toc.setObjectName("opt_pdf_add_toc")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.SpanningRole, self.opt_pdf_add_toc)
        self.opt_toc_title = QtWidgets.QLineEdit(Form)
        self.opt_toc_title.setObjectName("opt_toc_title")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.FieldRole, self.opt_toc_title)
        self.label_14 = QtWidgets.QLabel(Form)
        self.label_14.setObjectName("label_14")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.LabelRole, self.label_14)
        self.label.setBuddy(self.opt_paper_size)
        self.label_3.setBuddy(self.opt_custom_size)
        self.label_11.setBuddy(self.opt_unit)
        self.label_4.setBuddy(self.opt_pdf_serif_family)
        self.label_5.setBuddy(self.opt_pdf_sans_family)
        self.label_6.setBuddy(self.opt_pdf_mono_family)
        self.label_7.setBuddy(self.opt_pdf_standard_font)
        self.label_8.setBuddy(self.opt_pdf_default_font_size)
        self.label_9.setBuddy(self.opt_pdf_mono_font_size)
        self.label_12.setBuddy(self.opt_pdf_header_template)
        self.label_13.setBuddy(self.opt_pdf_footer_template)
        self.label_14.setBuddy(self.opt_toc_title)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.label_10.setText(_("<b>Note:</b> The paper size settings below only take effect if you enable the \"Override\" checkbox below. Otherwise the size from the output profile will be used."))
        self.opt_override_profile_size.setText(_("&Override paper size set in output profile"))
        self.label.setText(_("&Paper Size:"))
        self.label_3.setText(_("&Custom size:"))
        self.label_11.setText(_("&Unit:"))
        self.opt_preserve_cover_aspect_ratio.setText(_("Preserve &aspect ratio of cover"))
        self.opt_pdf_page_numbers.setText(_("Add page &numbers to the bottom of every page"))
        self.label_4.setText(_("Se&rif family:"))
        self.label_5.setText(_("&Sans family:"))
        self.label_6.setText(_("&Monospace family:"))
        self.label_7.setText(_("S&tandard font:"))
        self.label_8.setText(_("Default font si&ze:"))
        self.opt_pdf_default_font_size.setSuffix(_(" px"))
        self.label_9.setText(_("Monospace &font size:"))
        self.opt_pdf_mono_font_size.setSuffix(_(" px"))
        self.groupBox.setTitle(_("Page headers and footers"))
        self.label_2.setText(_("You can insert headers and footers into every page of the produced PDF file by using header and footer templates. For examples, see the <a href=\"http://manual.calibre-ebook.com/conversion.html#converting-to-pdf\">documentation</a>."))
        self.label_12.setText(_("&Header template:"))
        self.label_13.setText(_("&Footer template:"))
        self.opt_pdf_add_toc.setText(_("Add a printable &Table of Contents at the end"))
        self.label_14.setText(_("&Title for ToC:"))

